"""\
This file is an installation script for CMFForum (ZPT skins).  It's meant
to be used as an External Method.  To use, add an external method to the
root of the CMF Site that you want ZPT skins registered in with the
configuration:

 id: install_forum
 title: Install Forum Product *optional*
 module name: CMFForum.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Test' tab.  The install function will execute and give
information about the steps it took to register and install the
ZPT skins into the CMF Site instance. 
"""

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.ExternalMethod import ExternalMethod

from Acquisition import aq_base
from cStringIO import StringIO
import string
import string 

from Acquisition import Implicit
import Persistence

from Products.CMFForum import cmfforum_globals, Forum, Post

__version__='0.1'

def install_SubSkin(self, outStream, skinFolder):
    """ Installs a subskin, should be just 1 folder.
    """
    skinstool=getToolByName(self, 'portal_skins')
    for skin in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )
        if not skinFolder in path:
            try:
                path.insert( path.index( 'custom')+1, skinFolder )
            except ValueError:
                path.append(skinFolder)
            path = string.join( path, ', ' )
            skinstool.addSkinSelection( skin, path )
            outStream.write('CMFForum subskin successfully installed into %s.\n' % skin)    
        else:
            outStream.write('CMFForum subskin was already installed into %s.\n' % skin)    

def install_ForumSkins(self, out):
    """
        Add a new skin, 'Plone', copying 'ZPT', if it exists, and then
        add our directories only to it.
    """

    skinstool = getToolByName(self, 'portal_skins')

    try:
        install_SubSkin(self, out, 'zpt_forum')
        out.write('CMFForum subskin successfully installed.\n')
    except:
        out.write('CMFForum subskin failed to install.\n')

    try:  
        addDirectoryViews( skinstool, 'skins', cmfforum_globals )
        out.write( "Added CMFForum directory views to portal_skins.\n" )
    except:
        out.write( 'Unable to add CMFForum directory view to portal_skins.\n')

def install(self):
    """ Register the Forum Skins with portal_skins and friends """
    skinstool = getToolByName(self, 'portal_skins')
    wf_tool = getToolByName(self, 'portal_workflow')

    out = StringIO()
    out.write( 'CMFForum installation tool v' + str(__version__) + '.\n')

    install_ForumSkins(self, out)

    # Register with the typestool manually instead of with manage_addTypeInformation
    # as the classes were not registered with utils.ContentInit in __init__.py
    types_tool = getToolByName(self, 'portal_types')
    for t in (Forum.factory_type_information,Post.factory_type_information):
        if t['id'] not in types_tool.objectIds():
            cfm = apply(ContentFactoryMetadata, (), t)
            types_tool._setObject(t['id'], cfm)
            out.write('Registered %s with the types tool\n' % t['id'])
        else:
            out.write('Skipping "%s" - already in types tool\n' % t['id'])

    # Remove workflows for Forum and Post
    wf_tool.setChainForPortalTypes( ('Forum','Post'), [])
    out.write('Removed workflow for Forum and Post')
            
    return out.getvalue()

def install_discussions(self):
    """ Replace default DiscussionTool from CMF by a CMFForum-based one """

    out = StringIO()
    out.write( 'CMFForum installation tool v' + str(__version__) + '.\n')

    install_SubSkin(self, out, 'zpt_forum/discussion')

    out.write( '* Replacing Default CMF DiscussionTool by CMFForums one.')
    self.manage_delObjects( 'portal_discussion' )
    addPloneTool=self.manage_addProduct['CMFForum'].manage_addTool
    addPloneTool('CMFForum Discussion Tool', None)
    out.write( 'Done.')

    return out.getvalue()



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from Interface import Base

class IPost(Base):
    def thread_top():
        """ returns the top ancestor """
    def ancestors():
        """Return a list of ancestral posts"""
    def children(sort_key='id', REQUEST=None):
        """Return a list of replies to this item"""
    def inReplyTo():
        """ Return the object for which this object is a reply. """
    def nextReply():
        """ Return the next object that is reply. """
    def nextMessage():
        """ Return the next object on the same level. """
    def previousMessage():
        """ Return the previous object on the same level. """
    def in_reply_to():
        """ Return the id of the object for which this object is a reply. """
    def childMessageCount():
        """ Counts the number of chilren of this message, including this message """
    def threadMessageCount():
        """ Looks at the topmost ancestor, and then counts children """
    def childNewestMessageDate():
        """ Looks through children and returns the newest message date """
    def threadNewestMessageDate():
        """ Looks at the topmost ancestor, and then recurses
	    through the messages finding the last created message
        """
    def hasAttachments():
        """ Boolean value to test if the message has any attachment """
    def getAttachments():
        """ Return a list of attachment objects """
    def addPost(id='', title='', text='', author='', email='', file=None, REQUEST=None):
        """ Add a Post to its container """

	
	
	
	
	    



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from Interface import Base
class IForum(Base):
    def posts(REQUEST=None):
        """ returns the top level postings """
    def getForum():
        """ returns the Forum object """
    def generate_id():
        """ returns a available id """
    def addPost(id='', title='', body='', author='', email='', in_reply_to='', REQUEST=None):
        """ adds a Posting to current Forum """
    def addForum(id, title='', REQUEST=None):
        """ constructs a new Forum in the container it is called in """



## Script (Python) "getObjSize"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=obj
##title=
##
size=''

try:
    if hasattr(obj, 'get_size'): 
        size=obj.get_size()

    if size:
        if size<1024:
            return '1 K'
        elif size>1048576:
            return '%.02f M' % float(size/1048576.0)
        else:
            return str(int(size)/1024)+' K'

except:
     return size

return size



## Script (Python) "forum_post_toSearch"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

dest = context.getForum().absolute_url() + '/forum_search_form'
context.REQUEST.RESPONSE.redirect(dest)



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import Globals
from Post import Post
from ForumPermissions import AddForumPosting, AddForum, ViewForum
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents, \
     ModifyPortalContent, AddPortalContent
from AccessControl import ClassSecurityInfo
from interfaces.Forum import IForum

factory_type_information = { 'id'             : 'Forum'
			     , 'portal_type'	: 'Forum'
                             , 'meta_type'      : 'Forum'
                             , 'description'    : """\
Forums hold threaded discussions."""
                             , 'icon'           : 'forum_icon.gif'
                             , 'product'        : 'CMFForum'
                             , 'factory'        : 'addForum'
                             , 'filter_content_types' : 1
                             , 'allowed_content_types' : ('Post', ) 
                             , 'immediate_view' : 'folder_edit_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'forum_listing'
                                  , 'permissions'   : (View,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'home'
                                  , 'name'          : 'Forum Home'
                                  , 'action'        : 'forum_listing'
                                  , 'permissions'   : (View,)
                                  , 'category'      : 'object'
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'folder_edit_form'
                                  , 'permissions'   : (ManageProperties,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'localroles'
                                  , 'name'          : 'Local Roles'
                                  , 'action'        : 'folder_localrole_form'
                                  , 'permissions'   : (ManageProperties,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'forum_post_message'
                                  , 'name'          : 'Post Message'
                                  , 'action'        : 'forum_post_message'
                                  , 'permissions'   : (AddForumPosting, )
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'forum_search'
                                  , 'name'          : 'Search Forum'
                                  , 'action'        : 'forum_search_form'
                                  , 'permissions'   :(View, )
                                  , 'category'      : 'object'
                                  }
                                )
                             }
                           
# we sub CMFCatalogAware first to avoid PortalFolder's noping of the
# cataloging methods
class Forum(CMFCatalogAware, SkinnedFolder, PortalContent):

    meta_type = 'Forum'
    __implements__= (IForum, )

    manage_options = SkinnedFolder.manage_options
    
    security = ClassSecurityInfo()

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, title='', description=''):
        "Edit attributes, then reindex."
        Forum.inheritedAttribute('edit')(self, title, description)        
        self.reindexObject()

    security.declareProtected(ViewForum, 'posts')
    def posts(self, REQUEST=None):
	"""Return the top-level posts"""
	if REQUEST is None:
	    REQUEST=self.REQUEST
        
	posts = []
        for child in self.contentValues(['Post']):
            posts.append(child)
	return posts

    children = posts

    security.declareProtected(ViewForum, 'getForum')
    def getForum(self):
        """
        Return the Forum object.
        """
        return self

    security.declarePrivate('generate_id')
    def generate_id(self):
        """Find an available ID"""
        id = DateTime().millis()
        str_id = 'msg_%s'
        while hasattr(self, str_id % id):
            id = DateTime().millis()
        
        return str_id % id

    # XXX it is errorprone that addPost is a class method and module method and
    # duplicated in Forum and Post.  Should use the Forum module method IMHO.
    security.declareProtected(AddForumPosting, 'addPost')
    addPost = Post.addPost

    security.declareProtected(AddForum, 'addForum')
    def addForum(self, id, title='', REQUEST=None):
        """Add a Forum object"""

        forum = Forum(id, title)
        self._setObject(id, forum)
        forum = getattr(self, id)
        forum._setPortalTypeName('Forum')
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)

addForum = Forum.addForum

Globals.InitializeClass(Forum)



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
""" Basic portal discussion access tool.

$Id: DiscussionTool.py,v 1.2 2002/09/06 18:34:46 dreamcatcher Exp $
"""

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.CMFCore.utils import _dtmldir
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import Expression

from Forum import Forum

class DiscussionNotAllowed( Exception ):
    pass

class DiscussionTool( UniqueObject, SimpleItem, ActionProviderBase ):

    id = 'portal_discussion'
    meta_type = 'CMFForum Discussion Tool'
    _actions = [ActionInformation(id='reply'
                                , title='Reply'
                                , action=Expression(
                text='string: ${object_url}/talkback/forum_post_message')
                                , condition=Expression(
                text='python: object is not None and ' +
                'portal.portal_discussion.isDiscussionAllowedFor(object)')
                                , permissions=('Reply to item',)
                                , category='object'
                                , visible=1
                                 )]

    security = ClassSecurityInfo()

    manage_options = (ActionProviderBase.manage_options +
                     ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                     , 
                     ) + SimpleItem.manage_options)

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainDiscussionTool', _dtmldir )

    #
    #   'portal_discussion' interface methods
    #

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return available actions via tool.
        """
        return self._actions

    security.declarePublic( 'overrideDiscussionFor' )
    def overrideDiscussionFor(self, content, allowDiscussion):
        """
            Override discussability for a per object basis or clear and let
            the site default override.
        """
        mtool = getToolByName( self, 'portal_membership' )
        if not mtool.checkPermission( CMFCorePermissions.ModifyPortalContent
                                    , content
                                    ):
            raise "Unauthorized"

        if allowDiscussion is None or allowDiscussion == 'None':
            if hasattr(content, 'allow_discussion'):
                del content.allow_discussion
        else:
            content.allow_discussion = int(allowDiscussion)

    security.declarePublic( 'getDiscussionFor' )
    def getDiscussionFor(self, content):
        """
            Return the talkback for content, creating it if need be.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed
            
        talkback = getattr( content, 'talkback', None )
        if not talkback:
            talkback = self._createDiscussionFor( content )
        
        return talkback

    security.declarePublic( 'isDiscussionAllowedFor' )
    def isDiscussionAllowedFor( self, content ):
        '''
            Returns a boolean indicating whether a discussion is
            allowed for the specified content.
        '''
        if hasattr( content, 'allow_discussion' ):
            return content.allow_discussion
        typeInfo = getToolByName(self, 'portal_types').getTypeInfo( content )
        if typeInfo:
            return typeInfo.allowDiscussion()
        return 0

    #
    #   Utility methods
    #
    security.declarePrivate( '_createDiscussionFor' )
    def _createDiscussionFor( self, content ):
        """
            Create the object that holds discussion items inside
            the object being discussed, if allowed.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        # This creates the Forum
        # instead of the regular DiscussionItemContainer
        content.talkback = Forum('talkback','Discussion')
        # Set type manually, as the Forum was not created by factory.
        content.talkback._setPortalTypeName('Forum')
        return content.talkback


InitializeClass( DiscussionTool )



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""
"""

import Forum, Post, DiscussionTool, ForumPermissions
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import ToolInit, ContentInit
from Products.CMFCore.DirectoryView import registerDirectory

# ADD_CONTENT_PERMISSION = CMFCorePermissions.AddPortalContent

# contentClasses = ( Forum.Forum
#                  , Post.Post
#                  )


# contentConstructors = ( Forum.addForum
#                       , Post.addPost
#                       )

# bases = contentClasses

# fti = ( Forum.factory_type_information,
#         Post.factory_type_information )

tools = ( DiscussionTool.DiscussionTool, )

# Make the skins available as DirectoryViews.
registerDirectory('skins', globals())
cmfforum_globals=globals()

def initialize( context ):

    ToolInit('CMFForum Discussion Tool', tools=tools,
                   product_name='CMFForum', icon='tool.gif',
                   ).initialize( context )

    ContentInit( 'CMFForum Content'
               , content_types=(Forum.Forum,)
               , permission=ForumPermissions.AddForum
               , extra_constructors=(Forum.addForum,)
               , fti=(Forum.factory_type_information, )
               ).initialize( context )

    # Register manually for each type instead of using utils.ContentInit, 
    # as they need different permissions.
    #     context.registerClass(Forum.Forum,
    #                           constructors = (Forum.addForum,),
    #                           permission = ForumPermissions.AddForum)

    context.registerClass(Post.Post,
                          constructors = (Post.addPost,),
                          permission = ForumPermissions.AddForumPosting)



from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import setDefaultRoles

# Gathering Event Related Permissions into one place
ViewForum = 'View Forum'
AddForumPosting = 'Add Forum posting'
AddForum = 'Add Forum'

# Set up default roles for permissions
setDefaultRoles(AddForumPosting,
                ('Manager', 'Member', 'Anonymous'))

setDefaultRoles(ViewForum,
                ('Manager', 'Member', 'Anonymous'))

setDefaultRoles(AddForum,
                ('Manager', 'Member'))



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import Globals, string
from ForumPermissions import AddForumPosting
from DateTime import DateTime
from OFS.Image import File, Image
from OFS.content_types import guess_content_type
from AccessControl import ClassSecurityInfo
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.Document import Document
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, \
     ListFolderContents, ModifyPortalContent, AddPortalContent
from Products.CMFForum.ForumPermissions import AddForumPosting, ViewForum
from interfaces.Post import IPost

factory_type_information =   { 'id'             : 'Post'
                             , 'portal_type'    : 'Post'
                             , 'meta_type'      : 'Post'
                             , 'description'    : """\
Posts are like Discussion items but they just can be added into Forums."""
                             , 'icon'           : 'icon_post.gif'
                             , 'product'        : 'CMFForum'
                            , 'factory'        : 'addPost'
                             , 'filter_content_types' : 1
                             , 'allowed_content_types' : ('Post', 'File', 'Image' ) 
                             , 'immediate_view' : 'metadata_edit_form'
                             , 'global_allow'   : 0                               
                             , 'actions'        :
                                ( { 'id'            : 'view' 
                                  , 'name'          : 'View'
                                  , 'action'        : 'post_view'
                                  , 'permissions'   : (
                                      View, )
                                  }
                                , { 'id'            : 'post_reply'
                                  , 'name'          : 'Reply'
                                  , 'action'        : 'post_post_message'
                                  , 'permissions'   : ('Add Forum posting', )
                                  }
                                , { 'id'            : 'post_post'
                                  , 'name'          : 'Post'
                                  , 'action'        : 'forum_post_message'
                                  , 'permissions'   : ('Add Forum posting', )
                                  }
                                , { 'id'            : 'forum_home'
                                  , 'name'          : 'Forum Home'
                                  , 'action'        : 'forum_post_toForum'
                                  , 'permissions'   : (View, )
                                  }
                                , { 'id'            : 'forum_search'
                                  , 'name'          : 'Search Forum'
                                  , 'action'        : 'forum_post_toSearch'
                                  , 'permissions'   : (View, )
                                  }
                                )
                             }

def generate_id(self):
    """Find an available ID"""
    id = DateTime().millis()
    str_id = 'msg_%s'
    while hasattr(self, str_id % id):
        id = DateTime().millis()
        
    return str_id % id

class Post(SkinnedFolder, Document):

    meta_type = 'Post'
    __implements__= (IPost, )
    manage_options = SkinnedFolder.manage_options
    security = ClassSecurityInfo()


    def __init__(self, id, title='', description='', text='', author='', email='', in_reply_to='', text_format='plain'):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.description = description
	self.author = author
	self.email = email
        self._edit( text=text, text_format=text_format )
        self.setFormat( text_format )
	#self.created = self.modified = DateTime()
	# DefaultDublinCoreImpl.Date() uses now expects modified to be used,
	# differently
	self.created = DateTime()

    security.declareProtected(ViewForum, 'thread_top')
    def thread_top(self):
        """ returns the top ancestor """
        if len(self.ancestors()) > 0:
                return self.ancestors()[-1]
        else:
                return self

    security.declareProtected(ViewForum, 'ancestors')
    def ancestors(self):
	"""Return a list of ancestral posts"""
	ancestors = []
	parent = self.aq_inner.aq_parent
	while parent.meta_type=='Post':
            ancestors.append(parent)
            parent = parent.aq_inner.aq_parent
	return ancestors

    security.declareProtected(ViewForum, 'children')
    def children(self, sort_key='id', REQUEST=None):
	"""Return a list of replies to this item"""
	if REQUEST is None:
	    REQUEST = self.REQUEST
	children = []

        for child in self.contentValues(['Post']):
            children.append(child)
        children.sort(lambda x, y, sort_key=sort_key: cmp(getattr(x, sort_key, None) , getattr(y, sort_key, None)))
	return children

    security.declareProtected(ViewForum, 'inReplyTo')
    def inReplyTo(self):
        """
        Return the object for which this object is a reply.
        """
        parent = self.aq_inner.aq_parent
        if parent.meta_type=='Post':
            return parent
        else:
            return None

    # XXX remove this, use nextReplies
    security.declareProtected(ViewForum, 'nextReply')
    def nextReply(self):
        """
        Return the next object that is reply.
        """
        ids = self.contentIds(['Post'])
        ids.sort()
        if ids:
            return getattr(self, ids[0], None)
        return None

    security.declareProtected(ViewForum, 'nextReplies')
    def nextReplies(self):
        """
        Return a sequence of objects that are replies.
        """
        ids = self.contentIds(['Post'])
        ids.sort()
        if ids:
            return map(lambda x,self=self:getattr(self, x), ids)
        return None

    # XXX remove this, use nextMessages
    security.declareProtected(ViewForum, 'nextMessage')
    def nextMessage(self):
        """
        Return the next object on the same level.
        """
        ids = self.aq_inner.aq_parent.contentIds(['Post'])
        ids.sort()
        index = ids.index(self.getId())
        next = index + 1
        if next < len(ids):
            return getattr(self, ids[next], None)
        return None

    security.declareProtected(ViewForum, 'nextMessages')
    def nextMessages(self):
        """
        Return a sequence of next objects on the same level.
        """
        ids = self.aq_inner.aq_parent.contentIds(['Post'])
        ids.sort()
        index = ids.index(self.getId())
        next = index + 1
        if next < len(ids):
            return map(lambda x,self=self:getattr(self, x), ids[next:])
        return None

    # XXX remove this, use previousMessages
    security.declareProtected(ViewForum, 'previousMessage')
    def previousMessage(self):
        """
        Return the previous object on the same level.
        """
        ids = self.aq_inner.aq_parent.contentIds(['Post'])
        ids.sort()
        index = ids.index(self.getId())
        previous = index - 1
        if previous >= 0:
            return getattr(self, ids[previous], None)
        return None

    security.declareProtected(ViewForum, 'previousMessages')
    def previousMessages(self):
        """
        Return a sequence of previous objects on the same level.
        """
        ids = self.aq_inner.aq_parent.contentIds(['Post'])
        ids.sort()
        ids.reverse()
        index = ids.index(self.getId())
        previous = index + 1
        if previous >= 0:
            return map(lambda x,self=self:getattr(self, x), ids[previous:])
        return None

    security.declareProtected(ViewForum, 'in_reply_to')
    def in_reply_to(self):
        """
        Return the id of the object for which this object is a reply.
        """
        parent = self.aq_inner.aq_parent
        if parent.meta_type=='Post':
            return parent.getId()
        else:
            return ''

    security.declareProtected(ViewForum, 'childMessageCount')
    def childMessageCount(self):
        """ Counts the number of chilren of this message, including this message"""
        count=1 # this message
        for message in self.children():
                count = count + message.childMessageCount()
        return count

    security.declareProtected(ViewForum, 'threadMessageCount')
    def threadMessageCount(self):
        """ Looks at the topmost ancestor, and then counts children"""

        message_count = self.thread_top().childMessageCount()
        return message_count

    security.declareProtected(ViewForum, 'childNewestMessageDate')
    def childNewestMessageDate(self):
        """ Looks through children and returns the newest message date"""
        newest=self.created
        for message in self.children():
                test_date = message.childNewestMessageDate()
                if test_date > newest:
                       newest = test_date
        return newest

    security.declareProtected(ViewForum, 'threadNewestMessageDate')
    def threadNewestMessageDate(self):
        """ Looks at the topmost ancestor, and then recurses
            through the messages finding the last created message"""
        newest=self.thread_top().childNewestMessageDate()
        return newest

    security.declareProtected(ViewForum, 'hasAttachments')
    def hasAttachments(self):
        """
        Boolean value to test if the message has any attachment.
        """
        return self.contentIds(['Portal File', 'Portal Image'])

    security.declareProtected(ViewForum, 'getAttachments')
    def getAttachments(self):
        """
        Return a list of attachment objects.
        """
        return self.contentValues(['Portal File', 'Portal Image'])

    security.declareProtected(AddForumPosting, 'addPost')
    def addPost(self, id='', title='', text='', author='', email='',
                file=None, REQUEST=None, text_format='structured-text'):
        """Add a Post to its container"""

        if not id:
            id = generate_id(self)
        item = Post(id, title=title, text=text, author=author, email=email,
                    text_format=text_format)
        self._setObject(id, item)
        post = getattr(self, id)
        post._setPortalTypeName('Post')

        if file and file.filename:
	    id = string.split(file.filename, "\\")[-1]
            body = file.read()
            content_type = file.headers['Content-Type']
            if guess_content_type(id, body)[0][0:5] == 'image':
                post.invokeFactory( 'Image', id=id, file=file,
                                    content_type=content_type)
            else:
                post.invokeFactory( 'File', id=id, file=file,
                                    content_type=content_type)

        if REQUEST is not None:
            return self.manage_main(self, REQUEST)

addPost = Post.addPost

Globals.InitializeClass(Post)



## Script (Python) "post_add"
##bind container=container
##bind context=context
##bind namespace=_
##bind script=script
##bind subpath=traverse_subpath
##parameters=author='', email='', text='', title='', submit='', file=''
##title=Add a new Post
##
new_id='msg_' + str(int( context.ZopeTime()))
context.addPost(id=new_id, title=title, email=email, author=author, text=text, file=file)
url = '%s/%s' % (context.absolute_url(), context.getTypeInfo().getActionById('view',''))
return context.REQUEST.RESPONSE.redirect(url)



## Script (Python) "queryCatalog"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=wraps the portal_catalog with a rules qualified query
##
results=[]
REQUEST=context.REQUEST
catalog=context.portal_catalog
indexes=catalog.indexes()
query={}

for i in indexes:
    v=REQUEST.get(i, None)
    if v:
        query.update({i:v})

notRawIndexFields=[k for k in REQUEST.form.keys() if k not in query.keys()]
if notRawIndexFields:
    for k in notRawIndexFields:
        if k.endswith('_usage'): 
            query.update({k:REQUEST.get(k)})

if query:
    results=catalog(query)
return results
